﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.SearchModels;
using PizzeriaContracts.StoragesContracts;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using PizzeriaFileImplement.models;

namespace PizzeriaFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;
        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            return source.Shops
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return source.Shops
                .Where(x => x.ShopName.Contains(model.ShopName))
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Shops
            .FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.ShopName) && x.ShopName == model.ShopName) ||
                (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public bool TrySellPizza(IPizzaModel pizza, int count)
        {
            int hasCount = 0;

            source.Shops.ForEach(x =>
            {
                if (x.ShopPizzas.TryGetValue(pizza.Id, out var pair))
                {
                    hasCount += pair.Item2;
                }
            });

            if (hasCount < count) return false;

            source.Shops.ForEach(x =>
            {
                if (x.ShopPizzas.TryGetValue(pizza.Id, out var pair))
                {
                    if(count >= pair.Item2)
                    {
                        count -= pair.Item2;
                        x.ShopPizzas[pizza.Id] = (pizza, 0);
                    } else
                    {
                        x.ShopPizzas[pizza.Id] = (pizza, pair.Item2 - count);
                        count = 0;
                    }

                    x.Update(new ShopBindingModel
                    {
                        Id = x.Id,
                        Addres = x.Addres,
                        Capacity = x.Capacity,
                        OpenTime = x.OpenTime,
                        ShopName = x.ShopName,
                        ShopPizzas = x.ShopPizzas
                    });
                }
            });

            source.SaveShops();
            return true;
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Shops.Remove(element);
                source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
