﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using System.Xml.Linq;

namespace PizzeriaFileImplement.models
{
    public class Shop : IShopModel
    {
        public int Id { get; set; }
        public string ShopName { get; set; } = string.Empty;
        public string Addres { get; set; } = string.Empty;
        public DateTime OpenTime { get; set; }
        public int Capacity { get; set; }
        public Dictionary<int, int> Pizzas { get; private set; } = new();
        private Dictionary<int, (IPizzaModel, int)>? _shopPizzas = null;
        public Dictionary<int, (IPizzaModel, int)> ShopPizzas
        {
            get
            {
                if (_shopPizzas == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopPizzas = Pizzas.ToDictionary(x => x.Key, y =>
                    ((source.Pizzas.FirstOrDefault(z => z.Id == y.Key) as IPizzaModel)!,
                    y.Value));
                }
                return _shopPizzas;
            }
        }

        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Addres = model.Addres,
                OpenTime = model.OpenTime,
                Capacity = model.Capacity,
                Pizzas = model.ShopPizzas.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }

            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Addres = element.Element("Addres")!.Value,
                OpenTime = DateTime.Parse(element.Element("OpenTime")!.Value),
                Capacity = Convert.ToInt32(element.Element("Capacity")!.Value),
                Pizzas = element.Element("ShopPizzas")!.Elements("ShopPizza")
                    .ToDictionary(
                        x => Convert.ToInt32(x.Element("Key")?.Value),
                        x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        public void Update(ShopBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Addres = model.Addres;
            OpenTime = model.OpenTime;
            Capacity = model.Capacity;
            Pizzas = model.ShopPizzas.ToDictionary(x => x.Key, x => x.Value.Item2);
            _shopPizzas = null;
        }
        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Addres = Addres,
            OpenTime = OpenTime,
            Capacity = Capacity,
            ShopPizzas = ShopPizzas
        };
        public XElement GetXElement => new("Shop",
        new XAttribute("Id", Id),
        new XElement("ShopName", ShopName),
        new XElement("Addres", Addres),
        new XElement("OpenTime", OpenTime),
        new XElement("Capacity", Capacity),
        new XElement("ShopPizzas", Pizzas.Select(x =>
        new XElement("ShopPizza",
        new XElement("Key", x.Key),
        new XElement("Value", x.Value)))
        .ToArray()));
    }
}
